#!/bin/bash

#
# Data
#
sc=/pfs/lustrep2/scratch/project_462000125/samantao/mlperf-data

#
# Load 
#
module purge
module load CrayEnv
module load PrgEnv-cray/8.3.3
module load craype-accel-amd-gfx90a
module load rocm/5.2.3.lua

#
# Activate the AWS plugin.
#
module use /pfs/lustrep2/projappl/project_462000125/samantao-public/mymodules
module load aws-ofi-rccl/rocm-5.2.3.lua

source /flash/project_462000125/samantao/lumi-training/miniconda3/bin/activate mlperf
export LD_LIBRARY_PATH=$CONDA_PREFIX/lib:$LD_LIBRARY_PATH 

# Omnitrace environment
#
module use /pfs/lustref1/flash/project_462000125/samantao/lumi-training/omnitools/omnitrace-1.10.0-opensuse-15.3-ROCm-50200-PAPI-OMPT-Python3/share/modulefiles
module load omnitrace
# export OMNITRACE_CONFIG_FILE=$(pwd)/omnitrace.cfg

module use /flash/project_462000125/samantao/lumi-training/omnitools/omniperf3-modulefiles
module load omniperf
export ROOFLINE_BIN=/pfs/lustref1/flash/project_462000125/samantao/lumi-training/omnitools/omniperf3-install/bin/utils/lines/line-sle15sp3-mi200-rocm5


rm -rf run-trans.sh
cat > run-trans.sh << EOF
#!/bin/bash -ex

  if [ \$SLURM_LOCALID -eq 0 ] ; then
    rm -rf /dev/shm/*
    rocm-smi || true
  else
    sleep 2
  fi
  
  echo "Rank \$SLURM_PROCID --> \$(taskset -p \$\$)"
  
  export NCCL_SOCKET_IFNAME=hsn0,hsn1,hsn2,hsn3
  
  export MASTER_ADDR=\$(scontrol show hostname "\$SLURM_NODELIST" | head -n1)
  export MASTER_PORT=29500
  export WORLD_SIZE=\$SLURM_NPROCS
  export RANK=\$SLURM_PROCID
  
  echo "--> MASTER_ADDR: \$MASTER_ADDR"
  echo "--> MASTER_PORT: \$MASTER_PORT"
  echo "--> WORLD_SIZE: \$WORLD_SIZE"
  echo "--> RANK: \$RANK" 
  
  # export NCCL_DEBUG=INFO 
  # export NCCL_DEBUG_SUBSYS=INIT
  
  pcmd='python -u'
  
  if [ \$SLURM_PROCID -eq 0 ] ; then
  
    #
    # Clean old files
    #
    rm -rf workloads omnitrace-* results.*
  
    pcmd="omniperf profile -n pytorch --device 0  -- $(which python) -u "
    
  fi
  
  \$pcmd \
  ./train.py \
    --distributed-world-size \$WORLD_SIZE \
    $sc/data-transformer/transformer-mlperf-data/wmt14_en_de/utf8 \
    --seed 2028 \
    --arch transformer_wmt_en_de_big_t2t \
    --share-all-embeddings \
    --optimizer adam \
    --adam-betas '\\(0.9,0.997\\)' \
    --adam-eps 1e-9 \
    --clip-norm 0.0 \
    --lr-scheduler inverse_sqrt \
    --warmup-init-lr 0.0 \
    --warmup-updates 1000 \
    --lr 1.976e-3 \
    --min-lr 0.0 \
    --dropout 0.1 \
    --weight-decay 0.0 \
    --criterion label_smoothed_cross_entropy \
    --label-smoothing 0.1 \
    --max-tokens 10240 \
    --max-epoch 1 \
    --target-bleu 25.0 \
    --ignore-case \
    --no-save \
    --update-freq 1 \
    --fp16 \
    --seq-len-multiple 2 \
    --source_lang en \
    --target_lang de \
    --bucket_growth_factor 1.035 \
    --batching_scheme v0p5_better \
    --batch_multiple_strategy dynamic \
    --fast-xentropy \
    --max-len-a 1 \
    --max-len-b 50 \
    --lenpen 0.6 \
    --no-progress-bar \
    --dataloader-num-workers 2 \
    --enable-dataloader-pin-memory \
    --multihead-attn-impl fast_with_lyrnrm_and_dropoutadd \
    --dwu-num-blocks 4 \
    --dwu-num-rs-pg 2 \
    --dwu-num-ar-pg 2 \
    --dwu-num-ag-pg 0 \
    --dwu-overlap-reductions \
    --dwu-num-chunks 1 \
    --dwu-flat-mt \
    --dwu-compute-L2-grad-norm \
    --max-source-positions 64 \
    --max-target-positions 64 \
    --adam-betas '\\(0.9,0.98\\)' |& tee trans-\$SLURM_PROCID.log
EOF
chmod +x run-trans.sh

Nodes=$1

#
# An example allocation command
# 
#salloc -p gpu --exclusive -N $Nodes --gpus $((Nodes*8)) -t 2:00:00  --account project_462000075

#
# Capture SLURM ID in case I want to run the script in a different terminal
#
MYSLURMID=$(squeue --me | tail -n 1 | awk '{print $1;}')

c=fe
#
# Bind mask for one thread per core
#
MYMASKS1="0x${c}000000000000,0x${c}00000000000000,0x${c}0000,0x${c}000000,0x${c},0x${c}00,0x${c}00000000,0x${c}0000000000"
#
# Bind mask for two threads per core
#
MYMASKS2="0x${c}00000000000000${c}000000000000,0x${c}00000000000000${c}00000000000000,0x${c}00000000000000${c}0000,0x${c}00000000000000${c}000000,0x${c}00000000000000${c},0x${c}00000000000000${c}00,0x${c}00000000000000${c}00000000,0x${c}00000000000000${c}0000000000"

srun \
    --jobid=${MYSLURMID} \
    -N $Nodes \
    -n $(($Nodes*1)) \
    --cpu-bind=mask_cpu:$MYMASKS2 \
    --gpus $(($Nodes*8)) \
    ./run-trans.sh |& tee trans-all-ranks-$Nodes.log
